@@ -4,6 +4,8 @@ import os
 import sys
 import traceback
 
+import httpx
+
 from vectorcode import __version__
 from vectorcode.cli_utils import (
     CliAction,
@@ -100,8 +102,12 @@ async def async_main():
                 from vectorcode.subcommands import files
 
                 return_val = await files(final_configs)
-    except Exception:
+    except Exception as e:
         return_val = 1
+        if isinstance(e, httpx.RemoteProtocolError):  # pragma: nocover
+            e.add_note(
+                f"Please verify that {final_configs.db_url} is a working chromadb server."
+            )
         logger.error(traceback.format_exc())
     finally:
         await ClientManager().kill_servers()
