@@ -52,6 +52,16 @@ def query_result():
     }
 
 
+@pytest.fixture(scope="function")
+def empty_query_result():
+    return {
+        "ids": [],
+        "distances": [],
+        "metadatas": [],
+        "documents": [],
+    }
+
+
 @pytest.fixture(scope="function")
 def query_chunks():
     return ["query chunk 1", "query chunk 2"]
@@ -97,6 +107,15 @@ async def test_naive_reranker_rerank(naive_reranker_conf, query_result):
         assert isinstance(path, str)
 
 
+@pytest.mark.asyncio
+async def test_naive_reranker_rerank_empty_result(
+    naive_reranker_conf, empty_query_result
+):
+    reranker = NaiveReranker(naive_reranker_conf)
+    result = await reranker.rerank(empty_query_result)
+    assert len(result) == 0
+
+
 @patch("sentence_transformers.CrossEncoder")
 def test_cross_encoder_reranker_initialization(mock_cross_encoder: MagicMock, config):
     model_name = config.reranker_params["model_name_or_path"]
